"use strict";
// The MIT License (MIT)
//
// Copyright (c) 2020 The Prometheus Authors
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.
Object.defineProperty(exports, "__esModule", { value: true });
const lezer_promql_1 = require("lezer-promql");
class Matcher {
    constructor(type, name, value) {
        this.type = type;
        this.name = name;
        this.value = value;
    }
    matchesEmpty() {
        switch (this.type) {
            case lezer_promql_1.EqlSingle:
                return this.value === '';
            case lezer_promql_1.Neq:
                return this.value !== '';
            default:
                return false;
        }
    }
}
exports.Matcher = Matcher;
function createMatcher(labelMatcher, state) {
    const matcher = new Matcher(0, '', '');
    labelMatcher.iterate({
        enter: (type, start, end) => {
            switch (type.id) {
                case lezer_promql_1.LabelMatcher:
                    return undefined;
                case lezer_promql_1.LabelName:
                    matcher.name = state.sliceDoc(start, end);
                    break;
                case lezer_promql_1.MatchOp:
                    return undefined;
                case lezer_promql_1.EqlSingle:
                case lezer_promql_1.Neq:
                case lezer_promql_1.EqlRegex:
                case lezer_promql_1.NeqRegex:
                    matcher.type = type.id;
                    break;
                case lezer_promql_1.StringLiteral:
                    matcher.value = state.sliceDoc(start, end).slice(1, -1);
                    break;
            }
            return false;
        },
    });
    return matcher;
}
function buildLabelMatchers(labelMatchers, state) {
    const matchers = [];
    labelMatchers.forEach((value) => {
        matchers.push(createMatcher(value, state));
    });
    return matchers;
}
exports.buildLabelMatchers = buildLabelMatchers;
//# sourceMappingURL=matcher.js.map